/******************************************************************************
**                                                                           **
**      (c) Copyright 1990, Robert D. Goodearl  All Rights Reserved          **
**                                                                           **
**      goodearl@world.std.com                                               **
**                                                                           **
**      Written to compile on the Zortech C++ compiler V2.1                  **
**                                                                           **
*******************************************************************************
**                                                                           **
**  This software may be copied and used for non-commercial purposes         **
**  provided this header and all copyright notices are retained in the       **
**  source, and that the copyright notice remains in the executable.         **
**  This software is provided on an as-is basis with no warentees of         **
**  any kind.                                                                **
**                                                                           **
******************************************************************************/
#include <dos.h>
#include <stdlib.h>
#include "argopt.hpp"
#include "filespec.hpp"

/*************************************************************************
**      FileSpec::FileSpec()                                            **
**                                                                      **
**  This is the constructor for class FileSpec                          **
**************************************************************************
**  Input:  arglist     an array of char arrays containing arguments    **
**          argcount    count of arglist arrays                         **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
FileSpec::FileSpec(char *arglist[], int argcount)
{
    DBUGANN(DBFSpec,printf("f+");)
    ArgCount = argcount;
    ArgList  = arglist;
    ArgIndex = 0;
    DrvList  = 0L;
    Wild      = FALSE;
    DefaultWild = FALSE;
    ValidSpec   = FALSE;

    if (!ArgCount || (*ArgList[ArgIndex] == '\0'))
    {
        unsigned drv;

        DefaultWild = TRUE;
	ValidSpec   = TRUE;
        *(String*)this = "*.*";

	dos_getdrive(&drv);
	DrvList |= (OneLong << drv);
    }
    else
    {
        SetNext();
    }
}

/*************************************************************************
**      FileSpec::FileSpec()                                            **
**                                                                      **
**  This is the null constructor for class FileSpec                     **
**************************************************************************
**  Input:  none                                                        **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:  the += or = operator must be used to set a current file spec**
**          before an object created with this constructor is used.     **
**                                                                      **
*************************************************************************/
FileSpec::FileSpec()
{
    DBUGANN(DBFSpec,printf("f+");)
    ArgCount = 0;
    ArgList  = NULL;
    ArgIndex = 0;
    DrvList  = 0L;
    Wild      = FALSE;
    DefaultWild = FALSE;
    ValidSpec   = FALSE;
}

/*************************************************************************
**      void FileSpec::SetSpecType()                                    **
**                                                                      **
**  This function is a private function for the class that sets up the  **
**  current file spec                                                   **
**************************************************************************
**  Input:  none                                                        **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
void FileSpec::SetSpecType(void)
{
    int len;

    if (!ValidSpec)
    {
        fprintf(stderr, "SetSpecType: Invalid File Spec\n");
	exit(1);
    }

    //Here's where we fix up this argument to provide the results
    //we want.
    String & myfsp = *(String *)this;
    len = myfsp.StrLen();
    if (!len)
    {
	fprintf(stderr, "Zero length argument\n");
	exit(1);
    }
    for (int x=0; x<len; x++)
    {
        if ((myfsp[x] == '*') || (myfsp[x] == '?'))
	    Wild = TRUE;  //Explicit wildcard argument
    }
    //if string ends in asterisk, check to see if at end of extension
    if (myfsp[len-1] == '*')
    {
        //if only 1 char, this must be first part of string
	if (len == 1)
	{
	    myfsp += ".*";
	    Wild = TRUE;  //Explicit wildcard argument
	}
	else
	{
	    //check previous chars for dot
            for (x=len-2; x>=0; x--)
	    {
                if (myfsp[x] == '.')
		break;
	    }
	    //if dot not found, add asterisk for extension
	    if (x < 0)
	    {
	        myfsp += ".*";
	        Wild = TRUE;  //Explicit wildcard argument
	    }
	}
    }
    else if (len == 1)
    {
	if (myfsp[0] == '.')
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "\\*.*";
	}
        else if (myfsp[len-1] == '\\')
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "*.*";
	}
    }
    else
    {
        if (myfsp[len-1] == '\\')
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "*.*";
	}
	else if (myfsp[len-1] == ':')
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "*.*";
	}
	else if ((myfsp[len-1] == '.') && (myfsp[len-2] == '.'))
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "\\*.*";
	}
	else if ((myfsp[len-1] == '.') && (myfsp[len-2] == '\\'))
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "*.*";
	}
	else if ((myfsp[len-1] == '.') && (myfsp[len-2] == ':'))
	{
            DefaultWild = TRUE; //This is a directory spec
	    myfsp += "\\*.*";
	}
    }
    if ((len < 2) || (myfsp[1] != ':'))
    {
        unsigned drv;
	dos_getdrive(&drv);
	DrvList |= (OneLong << drv);
    }
    else
    {
	DrvList |= (OneLong << (myfsp[0]-DrvXlateBase));
    }
}

/*************************************************************************
**      BOOL FileSpec::SetNext()                                        **
**                                                                      **
**  This function sets up the next file spec based on the list of       **
**  arguments                                                           **
**************************************************************************
**  Input:  none                                                        **
**                                                                      **
**  Return: TRUE    if there is another file spec available             **
**          FALSE   if no more arguments are available to produce a     **
**                  file spec                                           **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
BOOL FileSpec::SetNext(void)
{
    if (ArgCount)
    {
        DefaultWild = FALSE;
	Wild      = FALSE;
	ValidSpec   = TRUE;
        ArgCount--;
        ((String *)this)->operator = (ArgList[ArgIndex++]);

	SetSpecType();
	return TRUE;
    }
    else
    {
        return FALSE;
    }
}

/*************************************************************************
**      void FileSpec::EvalSpec()                                       **
**                                                                      **
**  This function evaluates the current file spec                       **
**************************************************************************
**  Input:  none                                                        **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
void FileSpec::EvalSpec(void)
{
    SetSpecType();
}

/*************************************************************************
**      FileSpec & FileSpec::operator =()                               **
**                                                                      **
**  This function sets the current file spec string to a new argument   **
**************************************************************************
**  Input:  newstring   string used to replace current file spec String **
**                                                                      **
**  Return: reference to this FileSpec                                  **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
FileSpec & FileSpec::operator =(const String &newstring)
{
    if (newstring.StrLen()) ValidSpec = TRUE;
    ((String *)this) = newstring;
    return *this;
}

/*************************************************************************
**      FileSpec & FileSpec::operator +=()                              **
**                                                                      **
**  This function appends to the current file spec string               **
**************************************************************************
**  Input:  newstring   string to be appended to current file spec      **
**                                                                      **
**  Return: reference to this file spec                                 **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
FileSpec & FileSpec::operator +=(const String &newstring)
{
    String *st = (String *)this;

    st->operator+=(newstring.pStr());
    if (newstring.StrLen()) ValidSpec = TRUE;
    return *this;
}

/*************************************************************************
**      void FileSpec::SetDir()                                         **
**                                                                      **
**  This function appends a backslash (if necessary) and "*.*" to the   **
**  current file spec to turn it into a directory file spec             **
**************************************************************************
**  Input:  none                                                        **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
void FileSpec::SetDir(void)
{
    int mylen = ((String)*this).StrLen();

    if ((*(String*)this)[mylen-1] != '\\')
    {
        (*(String *)this) += '\\';
    }
    (*(String *)this) += "*.*";
    DefaultWild = TRUE;
    Wild = FALSE;
}

/*************************************************************************
**      FileSpec::fprint1()                                             **
**                                                                      **
**  This function prints the current file spec                          **
**************************************************************************
**  Input:  fp      FILE struct used for output                         **
**          mask    char string used as mask for output                 **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
void FileSpec::fprint1(FILE *fp, char *mask)
{
    fprintf(fp, mask, Str());
}

/*************************************************************************
**      BasePath()                                                      **
**                                                                      **
**  This function strips the end of a string containing a file spec     **
**  and leaves the resultant base path                                  **
**************************************************************************
**  Input:  pathname    string to be modified                           **
**                                                                      **
**  Return: none                                                        **
**************************************************************************
**  Notes:                                                              **
**                                                                      **
*************************************************************************/
void BasePath(String & pathname)
{
    int len = pathname.StrLen();

    //Now get rid of everything up to the last backslash or colon
    while (len && (pathname[len-1] != '\\') && (pathname[len-1] != ':'))
    {
        pathname--;
	len--;
    }
    if (!len)   //If the result of pruning is nothing, it's here
    {
        pathname = ".\\";
    }
}
